-- Notecase Pro script (requires  Notecase Pro >= 3.1.5)
-- Script name: InsertSymbols.lua
-- Verson: 1.1
-- Description:
-- Presents a selection dialog from which the user can 
-- choose from and insert various symbols
-- 
-- This script's author, Paul E. Merrell, hereby waives all copyright 
-- and related or neighboring rights to this script, pursuant to the
-- Creative Commons CC0 Universal relinquishment of rights found at
-- http://creativecommons.org/publicdomain/zero/1.0/ 

-- make sure app version supports necessary scriptable program commands

nMaj, nMin, nRev = Nc_Config_GetAppVersion()
if (nMaj < 3 or (nMaj == 3 and nMin < 1) or (nMaj == 3 and nMin == 1 and nRev < 5)) then
	Nc_GUI_MessageBox("ERROR: Script requires NcPro version >= 3.1.5")
	return
end

--store IDs of current document and note as variables nDocID and strNoteID

nDocID = Nc_Doc_ID_GetCur()
strNoteID = Nc_Note_ID_GetCur(nDocID)

-- check to see if note is selected and abort the script if not

if strNoteID == "" then
	Nc_GUI_MessageBox("ERROR: A note must be selected")
	return
end

-- get the current cursor position in the Note Pane
-- and store as variables nFrom and nTo

nFrom, nTo = Nc_Note_SelectionRange_Get(nDocID, strNoteID)

-- abort the script with an error message if text is selected

if nFrom ~= nTo then
	Nc_GUI_MessageBox("ERROR: Text must not be selected   \nwhen using this script.")
 return
end
 
-- Define menu items

str0Ellipsis                = "… Ellipsis"
str1SectionSign             = "§ Section Sign"
str2TwoSectionSigns         = "§§ Two Section Signs"
str3ParagraphSign           = "¶ Paragraph Sign"
str4TwoParagraphSigns       = "¶¶ Two Paragraphs Signs"
str5EmDash                  = "— Em Dash"
str6EnDash                  = "– En Dash"
str7DegreeSign              = "° Degree Sign"
str8DegreesCelsiusSign      = "℃ Degrees Celsius Sign"
str9DegreesFahrenheitSign   = "℉ Degrees Fahrenheit Sign"
str10EuroSymbol             = "€ Euro Symbol"
str11CentSign               = "¢ Cent Sign"
str12PoundsSterlingSymbol   = "£ Pounds Sterling Symbol"
str13YenSymbol              = "¥ Yen Symbol"
str14CopyrightSymbol        = "© Copyright Symbol"
str15RegisteredSymbol       = "® Registered Symbol"
str16TrademarkSymbol        = "™ Trademark Symbol"
str17About                  = "About This Script"

-- show Insert Symbols selector GUI and store user selection as variable nSelection

nSelection = Nc_GUI_SelectionDlg("Insert Symbol", str0Ellipsis, str1SectionSign, str2TwoSectionSigns, str3ParagraphSign, str4TwoParagraphSigns, str5EmDash, str6EnDash, str7DegreeSign, str8DegreesCelsiusSign, str9DegreesFahrenheitSign, str10EuroSymbol, str11CentSign, str12PoundsSterlingSymbol, str13YenSymbol, str14CopyrightSymbol, str15RegisteredSymbol, str16TrademarkSymbol, str17About)


--  ===============================
-- 			SELECTION VALUES
--  ===============================

-- user chooses Cancel -- abort the script

if nSelection == -1 then
	return
end

-- user chooses str17About  -- Display About Message and abort script

if nSelection == 17 then
	Nc_GUI_MessageBox([[
	
	                ABOUT  THIS  SCRIPT
	
	InsertSymbols.lua v. 1.1
	By Paul E. Merrell

	InsertSymbols.lua is a NoteCase Pro script that
	presents a menu enabling the user to select a symbol 
	or pair of symbols that the script will insert in the 
	current note at the current cursor position. 
	
	This script is most handy when assigned to a keyboard 
	shortcut or Toolbar icon. To learn how to make such
	assignments, in the NoteCase Pro Help file read How To 
	> Scripts, Usage of > Launch Scripts Quicker.	

	If you would like me to add other Unicode characters
	to this script, please post to the NoteCase Pro 
	discussion group at http://groups.google.com/group/
	notecase-pro> and include a link to each desired 
	character's page located on the http://www.fileformat.
	info/unicode/char/search.htm web site.
	
	The latest version of this script and other free scripts can 
	be downloaded from http://virtual-sky.com/scripting.php.       
	]])
 return
end

-- user chooses str0Ellipsis

if nSelection == 0 then
	strSymbols = "…"
end

-- user chooses str1SectionSign

if nSelection == 1 then
	strSymbols = "&sect;"
end

-- user chooses str2TwoSectionSigns

if nSelection == 2 then
	strSymbols = "&sect;&sect;"
end

-- user chooses str3ParagraphSign

if nSelection == 3 then
	strSymbols = "¶"
end

-- user chooses str4TwoParagraphSigns

if nSelection == 4 then
	strSymbols = "¶¶"
end

-- user chooses str5EmDash

if nSelection == 5 then
	strSymbols = "&#8212;"
end

-- user chooses str6EnDash

if nSelection == 6 then
	strSymbols = "&#8211;"
end

-- user chooses str7DegreeSign

if nSelection == 7 then
	strSymbols = "&deg;"
end

-- user chooses str8DegreesCelsiusSign

if nSelection == 8 then
	strSymbols = "℃"
end

-- user chooses str9DegreesFahrenheitSign

if nSelection == 9 then
	strSymbols = "℉"
end

-- user chooses str10EuroSymbol

if nSelection == 10 then
	strSymbols = "&euro;"
end

-- user chooses str11CentSign

if nSelection == 11 then
	strSymbols = "&cent;"
end

-- user chooses str12PoundsSterlingSymbol

if nSelection == 12 then
	strSymbols = "&pound;"
end
	
-- user chooses str13YenSymbol

if nSelection == 13 then
	strSymbols = "&yen;"
end

-- user chooses str14CopyrightSymbol

if nSelection == 14 then
	strSymbols = "&copy;"
end

-- user chooses str15RegisteredSymbol

if nSelection == 15 then
	strSymbols = "&reg;"
end

-- user chooses str16TrademarkSymbol

if nSelection == 16 then
	strSymbols = "&trade;"
end

--  ===============================
--					INSERT SYMBOL(S)
--  ===============================

-- Insert symbol(s) and get character count of string
-- to be inserted stored as variable nCharCount

nCharCount = Nc_Note_Content_Set(nDocID, strNoteID, strSymbols, 1, nFrom, nTo)

-- add nCharCount to nFrom and nTo variable's values to 
-- get the desired cursor position's character offsets

nFrom = nFrom + nCharCount
nTo = nTo + nCharCount

-- move cursor to new position after inserted symbol(s)

Nc_Note_SelectionRange_Set(nDocID, strNoteID, nFrom, nTo)


