--[[
Notecase Pro script (requires  Notecase Pro >= 3.3.6.)
Script name: LinksManager.lua
Version 1.0

Description: Stand-alone menued script with options:                        

	* Copy Current Note's Link                  
	* Copy List's Links
	* Copy Children's Links
	* Copy Descendants' Links
	* Copy Branch's Links
	* Copy Links for Notes Linking Here
	* Copy Breadcrumb Links
	* Copy Ancestor Links
	* List Notes Linking Here
	* Make External Link
	* Help
 
RIGHTS: This script's author, Paul E. Merrell, hereby waives all 
copyright and related or neighboring rights to this script, 
pursuant to the Creative Commons CC0 Universal relinquishment of 
rights found at http://creativecommons.org/publicdomain/zero/1.0/ 

--]]

-- ===============================
--    BEGIN SCRIPT
-- ===============================

-- make sure app version supports necessary scriptable program commands
nMaj, nMin, nRev = Nc_Config_GetAppVersion()
if (nMaj < 3 or (nMaj == 3 and nMin < 1) or (nMaj == 3 and nMin == 3 and nRev < 6)) then
	Nc_GUI_MessageBox("ERROR: Script requires NcPro version >= 3.3.6.")
	return
end

-- ===============================
--    INITIAL MENU
-- ===============================

-- Define variables for selection dialog

	strDlgTitle  = "Links Manager"
	strMenuItem0 = "Copy Current Note's Link"
	strMenuItem1 = "Copy List's Links"
	strMenuItem2 = "Copy Children's Links"
	strMenuItem3 = "Copy Descendants' Links"
	strMenuItem4 = "Copy Branch Links"
	strMenuItem5 = "Copy Links for Notes Linking Here"
	strMenuItem6 = "Copy Breadcrumb Links"
	strMenuItem7 = "Copy Ancestor Links"
	strMenuItem8 = "List Notes Linking Here"
	strMenuItem9 = "Make External Link"
	strMenuItem10 = "Help"

-- get user's choice of action to perform and store as variable nSelection 
nSelection = Nc_GUI_SelectionDlg(strDlgTitle, strMenuItem0, strMenuItem1, strMenuItem2, strMenuItem3, strMenuItem4, strMenuItem5, strMenuItem6, strMenuItem7, strMenuItem8, strMenuItem9, strMenuItem10)

-- check to see if user canceled and if so abort the script
if nSelection == -1 then
	return
end

-- ==================================
--    BEGIN FUNCTION DEFINITIONS
-- ==================================

-- ==================================
--    LINES FUNCTION
--			(Written by Miroslav Rajcic)
-- ==================================

-- break string into the array
function lines(str)
  local t = {}
  local i, lstr = 1, #str
  while i <= lstr do
    local x, y = string.find(str, "\r?\n", i)
    if x then t[#t + 1] = string.sub(str, i, x - 1)
    else break
    end
    i = y + 1
  end
  if i <= lstr then t[#t + 1] = string.sub(str, i) end
  return t
end -- function

-- ================================
--    REFORMAT NOTE TITLE FUNCTION
-- ================================

function ReformatNoteTitle(strNoteTitle)

	-- in multi-line note titles, replace newlines
	-- with space to make single line title
	strNoteTitle = string.gsub(strNoteTitle, "\n", " ") 
	
	-- if they exist replace triple spaces in title with single space 
	strNoteTitle = string.gsub(strNoteTitle, "   ", " ")

	-- if they exist replace double spaces in note title with single space
	strNoteTitle = string.gsub(strNoteTitle, "  ", " ")
	
	-- store reformatted title as strReformattedTitle 
	strReformattedTitle = strNoteTitle

 return strReformattedTitle

end -- function

-- =============================================
-- CREATE LINK FUNCTION
-- 	separate function to simplify later support 
--  for relative and absolute links too
-- 
--  Nb., this function is *not* used in 
-- "Make External Link"
-- =============================================

function CreateLink(strReformattedTitle)

	-- create internal link and append to strLink with line break and newline
	strLink = strLink .. "<a href=\"#" .. strNoteID .. "\">" .. strReformattedTitle .. "</a><br>\n"

	return strLink
end -- function
		
-- =========================================================
--    COPY LIST LINKS FUNCTION
-- 			Copies list's links to clipboard then switches to 
--      Tree Pane and resets former current note as current 
-- =========================================================

function CopyListLinks(nDocID)

	-- switch to List mode
	Nc_App_List_Switch(1)

	-- create empty strLink variable
	strLink = ""

	-- get and store count of listed notes as variable nListCount
	nListCount = Nc_App_List_GetCount()

	-- for each note in the List Pane
	for i=1,nListCount do 
	
		-- get info on the note (note indexes start from 0)
		strNoteID    = Nc_App_List_Note_GetID(i-1)
		strNoteTitle = Nc_Note_Title_Get(nDocID, strNoteID)

		-- reformat note title using function
		ReformatNoteTitle(strNoteTitle)

		-- create and append link to strLink using function
		CreateLink(strReformattedTitle)
	end

 	-- write links to the clipboard
	Nc_Env_Clipboard_Set(strLink, 1)

	-- switch to Tree Pane
	Nc_App_List_Switch(0)

	-- Set former current note as current note
	Nc_Note_ID_SetCur(strCurNoteID)
	
end -- function

-- ==============================================
--    LIST ANCESTORS FUNCTION
-- 		if bAncestor == 1 lists ancestors
--  	if bAncestor == 0 lists breadcrumb's notes
-- ==============================================

function ListAncestors(bAncestor)

	-- if breadcrumb, include current note's ID
	if bAncestor == 0 then
	
		-- set current note's ID as variable strNoteID
		strNoteID = strCurNoteID
		
		-- if ancestors, then start with parent note
		else do 
	
			-- get parent note's ID and store as variable strNoteID
			strNoteID = Nc_Note_Parent_GetID(nDocID, strCurNoteID)
		end 
	end
	
	-- format strNoteID with newline as variable str
	str = strNoteID .. "\n"
		
		-- process all ancestors and prepend each note's ID to str with a newline
		while strNoteID ~= "" do
		str = Nc_Note_Parent_GetID(nDocID, strNoteID) .. "\n" .. str
						
		-- proceed to parent note
		strNoteID = Nc_Note_Parent_GetID(nDocID, strNoteID)
	end

	-- clear the list 
	Nc_App_List_Clear()
	
	-- break string into a table of individual lines using lines function
	local buf = lines(str) 

	-- get the count of note IDs in the table
	nSize = table.getn ( buf )
	
	-- for each note ID, add the note to the list pane
	for i=1, nSize do
		Nc_App_List_AddNote(buf[i])
	end 
	
end -- function

-- ================================================
--    LIST DESCENDANTS FUNCTION
-- 		function for listing descendants or children
--		
--		if bTarget == 0 lists children only
--		if bTarget == 1 lists all descendants
-- ================================================

function ListDescendants(strNoteID, bTarget) 
	
	-- get count of note's children
	local nChildCount = Nc_Note_ChildCount(nDocID, strNoteID)
	
	-- for each child get note ID and store as strChildID
	for i=1,nChildCount do
		local strChildID = Nc_Note_Child_GetID(nDocID, strNoteID, i-1)
		
		-- list the note
		Nc_App_List_AddNote(strChildID)
		
		-- if bTarget is 1 loop the function to list any descendants too
		if bTarget == 1 then
			ListDescendants(strChildID, 1)
		end

	end

end -- function

-- ===============================
--    BEGIN ACTIONS
-- ===============================

-- get IDs for current document/note
nDocID = Nc_Doc_ID_GetCur()
strCurNoteID = Nc_Note_ID_GetCur(nDocID)

-- ======================================================
--   COPY CURRENT NOTE'S INTERNAL LINK
-- 	user chose strMenuItem0 = "Copy Current Note's Link"
-- ======================================================

if nSelection == 0 then

	-- check if note is selected and if not abort with error message
	if strCurNoteID == "" then
			Nc_GUI_MessageBox("ERROR: No note selected, aborting")
		return
	end

	-- get note title and store as strNoteTitle	
	strNoteTitle = Nc_Note_Title_Get(nDocID, strCurNoteID)

	-- Reformat note title using function
	strReformattedTitle = ReformatNoteTitle(strNoteTitle)

	-- write link to clipboard
	Nc_Env_Clipboard_Set("<a href=\"#" .. strCurNoteID .. "\">" .. strReformattedTitle .. "</a>", 1)

 return
end

-- ================================================
--   COPY LINKS FOR ALL LISTED NOTES
-- User selected strMenuItem1, "Copy List's Links"
-- ================================================

if nSelection == 1 then

	-- Throw error message and abort if no note is currently selected		
	if strCurNoteID == "" then
		Nc_GUI_MessageBox("ERROR: No note selected, aborting")
	 return 
	end

	-- copy list links using function
	CopyListLinks(nDocID)

 return
end

-- =======================================================
--   COPY CHILDREN'S LINKS
--   User selected strMenuItem2 = "Copy Children's Links"
-- =======================================================

if nSelection == 2 then

	-- Throw error message and abort if no note is currently selected		
	if strCurNoteID == "" then
		Nc_GUI_MessageBox("ERROR: No note selected, aborting")
	 return 
	end

	-- set value of strNoteID same as strCurNoteID
	strNoteID = strCurNoteID
	
	-- clear the list pane
	Nc_App_List_Clear()

	-- list all child notes using using function
	ListDescendants(strCurNoteID, 0)

	-- copy list links to clipboard using function
	CopyListLinks(nDocID)

 return	
end

-- ==========================================================
--   COPY DESCENDANTS' LINKS
--   User selected 	strMenuItem3 = "Copy Descendants' Links"
-- ==========================================================

if nSelection == 3 then

	-- Throw error message and abort if no note is currently selected		
	if strCurNoteID == "" then
		Nc_GUI_MessageBox("ERROR: No note selected, aborting")
	 return 
	end

	-- clear the list pane
	Nc_App_List_Clear()
	
	-- set value of strNoteID same as strCurNoteID
	strNoteID = strCurNoteID

	-- list all descendant notes using using function
	ListDescendants(strNoteID, 1)

	-- copy list links to clipboard using function
	CopyListLinks(nDocID)

 return
end

-- ====================================================
--   COPY CURRENT BRANCH'S LINKS
--   User selected 	strMenuItem4 = "Copy Branch Links"
-- ====================================================

if nSelection == 4 then

	-- Throw error message and abort if no note is currently selected		
	if strCurNoteID == "" then
		Nc_GUI_MessageBox("ERROR: No note selected, aborting")
	 return 
	end

	-- clear the list pane
	Nc_App_List_Clear()
	
	-- add current note to the list
	Nc_App_List_AddNote(strCurNoteID)
	
	-- set value of strNoteID same as strCurNoteID
	strNoteID = strCurNoteID

	-- list all descendant notes using function
	ListDescendants(strNoteID, 1)

	-- copy list links to clipboard using function
	CopyListLinks(nDocID)
	
 return
end 

-- ==================================================================
--   COPY INTERNAL LINKS FOR NOTES LINKING HERE
--   User selected strMenuItem5, "Copy Links for Notes Linking Here"
-- ==================================================================

if nSelection == 5 then

	-- Throw error message and abort if no note is currently selected		
	if strCurNoteID == "" then
		Nc_GUI_MessageBox("ERROR: No note selected, aborting")
	 return 
	end

	-- clear the list pane
	Nc_App_List_Clear()
	
	-- get IDs for notes that link to the current note and store 
	-- as variable strNotesList (IDs are semicolon-delimited list) 
	strNotesList = Nc_NoteGroup_NotesLinkingToMe_Get(nDocID, strCurNoteID)

	-- replace semicolon delimiters with newlines 
	strNotesList = string.gsub(strNotesList, ";", "\n")

	-- break string into a table with individual lines using lines function
	local buf = lines(strNotesList) 

	-- get the count of note IDs in the table
	nSize = table.getn ( buf )

	-- for each note ID, add the note to the list pane
	for i=1, nSize do
		Nc_App_List_AddNote(buf[i])
	end 

	-- get list's links using function
	CopyListLinks(nDocID)

 return
end

-- =======================================================
--   COPY CURRENT NOTE'S BREADCRUMB LINKS
--   User selected 	strMenuItem6, "Copy Breadcrumb Links"
-- =======================================================

if nSelection == 6 then

	-- Throw error message and abort if no note is currently selected		
	if strCurNoteID == "" then
		Nc_GUI_MessageBox("ERROR: No note selected, aborting")
		return 
	end 

	-- clear the list pane
	Nc_App_List_Clear()
	
	-- List breadcrumb notes using function
	ListAncestors(0)

		-- copy list links using function
	CopyListLinks(nDocID)
	
 return
end

-- ======================================================
--   COPY CURRENT NOTE'S ANCESTOR LINKS
--   User selected 	strMenuItem7 = "Copy Ancestor Links"
-- ======================================================

if nSelection == 7 then

	-- Throw error message and abort if no note is currently selected		
	if strCurNoteID == "" then
		Nc_GUI_MessageBox("ERROR: No note selected, aborting")
		return 
	end 
	
	-- clear the list pane
	Nc_App_List_Clear()
	
	-- List ancestor notes using function
	ListAncestors(1)

		-- copy list links to clipboard using function
	CopyListLinks(nDocID)
	
 return
end

-- =========================================================
--   LIST NOTES LINKING TO THIS NOTE
--   User selected strMenuItem8, "List Notes Linking Here"
-- =========================================================

if nSelection == 8 then

	-- check to make sure note is selected and if not, abort with error message
	if strCurNoteID == "" then
		Nc_GUI_MessageBox("ERROR: No note selected, aborting")
		return 
	end

	-- clear the List Pane and switch to List Mode
	Nc_App_List_Clear()
	Nc_App_List_Switch(1)

	-- get IDs for notes that link to the current note and store 
	-- as variable strNotesList (IDs are semicolon-delimited list) 
	strNotesList = Nc_NoteGroup_NotesLinkingToMe_Get(nDocID, strCurNoteID)

	-- replace semicolon delimiters with newlines 
	strNotesList = string.gsub(strNotesList, ";", "\n")

	-- break string into a table with individual lines using lines function
	local buf = lines(strNotesList) 

	-- get the count of note IDs in the table
	nSize = table.getn ( buf )

	-- for each note ID, add the note to the list pane
	for i=1, nSize do
		Nc_App_List_AddNote(buf[i])
	end 

 return
end

-- =================================================
--	  MAKE EXTERNAL LINK
--	  user chose strMenuItem9, "Make External Link"
-- =================================================

if nSelection == 9 then

	-- check to see if note is selected and if not, throw 
	-- error message and abort the script
	if strCurNoteID == "" then
		Nc_GUI_MessageBox("ERROR: A note must be selected")
	 return
	end

	-- get character offsets for selected text and store as variables
	-- nSelFrom and nSelTo
	nSelFrom, nSelTo = Nc_Note_SelectionRange_Get(nDocID, strCurNoteID)

	-- abort the script with an error message if no text is selected
	if nSelFrom == nSelTo then
		Nc_GUI_MessageBox("ERROR: Text must be selected in the Note Pane     \nbefore running this script.")
	 return
	end

	-- get selected content in HTML format and store as variable strSelection
	strSelection = Nc_Note_Content_Get(nDocID, strCurNoteID, 0, nSelFrom, nSelTo)

	-- throw text input dialog for user to paste URL to 
	-- and store input as variable strURL
	strURL = Nc_GUI_InputDlg("Enter Text", "Enter or paste URL here") 

	-- assemble link as variable strLink
	strLink = "<a href=\"" .. strURL .. "\">" .. strSelection .. "</a>"
	
	-- replace the selected content in the note as HTML
	Nc_Note_Content_Set(nDocID, strCurNoteID, strLink, 1, nSelFrom, nSelTo)

	-- be certain that nSelTo equals the greater of nSelFrom and nSelTo
	if nSelFrom > nSelTo then
		nSelTo = nSelFrom
	end

	-- remove selection highlighting and set the cursor position 
	-- immediately after the inserted content
	Nc_Note_SelectionRange_Set(nDocID, strCurNoteID, nSelTo, nSelTo)

 return
end

-- ====================================
--	  HELP
--	  user chose strMenuItem10, "Help"
-- ====================================

if nSelection == 10 then

	strText = [[Copy Current Note's Link — 
  Copies to the clipboard the current note's internal link.

Copy List's Links — 
  Copies to the clipboard the internal links for all notes in the current list of notes, one link per line. 	   

Copy Children's Links — 
  Copies to the clipboard the internal links for the current note's children.

Copy Descendants' Links —
  Copies to the clipboard the internal links for the current note's descendants.

Copy Branch Links — 
  Copies to the clipboard the internal links for the current branch. 

Copy Links for Notes Linking Here —
  Copies to the clipboard the internal links for all notes  that link to the current note, one link per line.

Copy Breadcrumb Links — 
  Copies to the clipboard the internal links for the current note and all of its ancestor notes, one link per line.

Copy Ancestor Links — 
  Copies to the clipboard the internal links for all ancestor notes of the current note, one link per line.

List Notes Linking Here — 
  Lists all notes that link to the current note.

Make External Link — 
  Linkifies selected text using URL entered or pasted into input dialog

Help — 
  This documentation.
	
Version 1.0	
	]]
	
Nc_GUI_InfoBox(strText, 1, "Links Manager Help")	

 return
end
